
from math import *
import pickle

from itertools import izip,count
from OpenGL.GL import *

from cHero import Hero
from cTexture import Texture
from cTimer import Timer
from cFont import Font


class levelData:
	pass

class Tile:
	#VERTS_BETWEEN = [(-.5,.5,.5),(.5,.5,.5),(.5,1.5,.5),(-.5,1.5,.5)]
	VERTS_FLOOR = [ (-.5,-.5,.5),(.5,-.5,.5),(.5,-.5,-.5),(-.5,-.5,-.5)]
	VERTS_CEIL = [ (-.5,.5,-.5),(.5,.5,-.5),(.5,.5,.5),(-.5,.5,.5)]
	VERTS_BWALL = [ (-.5,-.5,-.5),(.5,-.5,-.5),(.5,.5,-.5),(-.5,.5,-.5)]
	VERTS_LWALL = [ (-.5,-.5,.5),(-.5,-.5,-.5),(-.5,.5,-.5),(-.5,.5,.5)]
	VERTS_RWALL = [ (.5,-.5,-.5),(.5,-.5,.5),(.5,.5,.5),(.5,.5,-.5)]
	VERTS = [VERTS_FLOOR,VERTS_CEIL,VERTS_LWALL,VERTS_BWALL,VERTS_RWALL]

	edit = 0

	def __init__(self):
		self.floor = None
		self.ceil = None
		self.lwall = None
		self.bwall = 0
		self.rwall = None

		self.solid = 0

		self.floortex = Texture.load('graphics/tiles_floor.png')
		self.ceiltex = Texture.load('graphics/tiles_ceil.png')
		self.walltex = Texture.load('graphics/tiles_wall.png')

		self.tilemaps = [   self.floortex,
							self.ceiltex,
							self.walltex,
							self.walltex,
							self.walltex ]

		self.id = [self.floor, self.ceil, self.lwall, self.bwall, self.rwall]


	def draw(self):
		if self.solid:
			self.drawSolid()
		else:	
			for t_map,t_id,verts in izip(self.tilemaps,self.id,self.VERTS):
				if t_id is None:
					continue
				self.drawTile(verts,t_map,t_id)
	def drawSolid(self):
		glColor3f(1,.7,1)
		glDisable(GL_TEXTURE_2D)
		glPushMatrix()
		glTranslatef(0,0,.5)
		glRectf(-.5,-.5,.5,.5)
		glPopMatrix()
			
	def drawTile(self,verts,t_map,t_id):
		t_map.setup()
		glBegin(GL_QUADS)
		for p,t in izip(verts,self.genTexCoords(t_id)):
			glTexCoord(*t); glVertex3f(*p)
		glEnd()

	def genTexCoords(self, t_id):
		x,y = (t_id%4,t_id/4)
		D = 128.0/512.0
		x *= D
		y *= D
		d = 1.0/512.0 # shrink the uv by 'd' to prevent glitches
		return [ (x+d,y+D-d),(x+D-d,y+D-d),(x+D-d,y+d),(x+d,y+d) ]

	def change(self,dir):
		id = self.id[self.edit]
		if id is None:
			if dir==1:
				id = 0
			elif dir == -1:
				id = 15
		else:
			id+=dir
			if id<0 or id>15:
				id = None
		print id
		self.id[self.edit] = id
	def toggleSolid(self):
		self.solid = (self.solid+1)%3

	def drawEditOverlay(self):
		editverts = [(0,0),(50,0),(50,50),(0,50)]

		for t_map,t_id,i in izip(self.tilemaps,self.id,count()):
			glPushMatrix()
			glTranslatef(i*50,0,0)
			if self.edit == i:
				glColor4f(0,0,1,1)
				glDisable(GL_TEXTURE_2D)
				glRecti(20,50,30,60)
			if t_id is None:
				glColor4f(1,0,0,1)
				glDisable(GL_TEXTURE_2D)
				glRecti(10,23,37,30)
			else:
				t_map.setup()
				glColor4f(1,1,1,1)
				glBegin(GL_QUADS)
				for v,t in izip(editverts,self.genTexCoords(t_id)):
					glTexCoord(*t); glVertex2i(*v)
				glEnd()
			glPopMatrix()
		glPushMatrix()
		glDisable(GL_TEXTURE_2D)
		glTranslatef(5*50,0,0)
		if self.solid == 0:
			glColor3f(.2,.2,.2)
		elif self.solid == 1:
			glColor3f(.6,.6,.6)
		elif self.solid == 2:
			glColor3f(1,1,1)
		glRecti(15,15,35,35)
		glPopMatrix()

class EditCursor:
	def __init__(self,level,pos):
		self.level = level
		self.pos = pos
		self.move(0,0)

	def move(self,x,y):
		newx = (self.pos[0]+x)%5
		newy = (self.pos[1]+y)%5
		self.pos = (newx,newy)
		self.activeTile = self.level.tmap[newx+newy*5]

	def draw(self):
		glPushMatrix()
		glTranslatef(self.pos[0],self.pos[1],1.2)
		glDisable(GL_TEXTURE_2D)
		glColor3f(.3,.6,.4)
		glRectf(-.04,-.5,.04,.5)
		glRectf(-.5,-.04,.5,.04)
		glPopMatrix()
	def change(self,dir):
		self.activeTile.change(dir)
	def changeEditTile(self,dir):
		Tile.edit = (Tile.edit+dir)%5
	def toggleSolid(self):
		self.activeTile.toggleSolid()
	
class Level:

	def __init__(self,levelfile):
		self.tileTex = Texture.load('graphics/tiles.png')
		self.liquidtex = Texture.load('graphics/meter.png')

		self.font  = Font('graphics/font.tga')

		self.timer = Timer(False)

		self.reset()

		self.tmap = []
		self.w = 0
		self.h = 0
		#self.tmap[0].solid = 1
		#self.tmap[1].solid = 1
		#self.tmap[2].solid = 1
		#self.tmap[3].solid = 1
		#self.tmap[4].solid = 1
		#self.tmap[5].solid = 1

		self.loadLevel()

		self.editMode = False
		self.editCursor = EditCursor(self,(0,0))

	def reset(self):
		self.hero = Hero( (3,3) )


		self.camPos = (3,3)
		self.camTarget = self.hero
		self.camSpeed = 0

	def drawmaptile(self, tilenum):
		if(tilenum==0):
			self.drawTile(Tile.VERTS_BWALL,((0,2)))
		elif(tilenum==1):
			self.drawTile(Tile.VERTS_CEIL,(0,3))
			self.drawTile(Tile.VERTS_BWALL,((0,2)))
		elif(tilenum==2):
			self.drawTile(Tile.VERTS_BWALL,((0,2)))
			self.drawTile(Tile.VERTS_FLOOR,(1,3))
		elif(tilenum==3):
			self.drawTile(Tile.VERTS_BETWEEN,(1,0))
		elif(tilenum==4):
			self.drawTile(Tile.VERTS_BWALL,((0,2)))
			self.drawTile(Tile.VERTS_LWALL,((1,2)))
		elif(tilenum==5):
			self.drawTile(Tile.VERTS_BWALL,((0,2)))
			self.drawTile(Tile.VERTS_LWALL,((1,2)))
			self.drawTile(Tile.VERTS_FLOOR,(1,3))
		elif(tilenum==6):
			self.drawTile(Tile.VERTS_BWALL,((0,2)))
			self.drawTile(Tile.VERTS_LWALL,((1,2)))
			self.drawTile(Tile.VERTS_CEIL,(0,3))
		elif(tilenum==7):
			self.drawTile(Tile.VERTS_BWALL,((0,2)))
			self.drawTile(Tile.VERTS_RWALL,((1,2)))

		elif(tilenum==8):
			self.drawTile(Tile.VERTS_BWALL,((0,2)))
			self.drawTile(Tile.VERTS_RWALL,((1,2)))
			self.drawTile(Tile.VERTS_FLOOR,(1,3))
		elif(tilenum==9):
			self.drawTile(Tile.VERTS_BWALL,((0,2)))
			self.drawTile(Tile.VERTS_RWALL,((1,2)))
			self.drawTile(Tile.VERTS_CEIL,(0,3))
			

	def drawLevel(self):
		t = self.timer.time/1000.0
		
		glEnable(GL_DEPTH_TEST)
		glEnable(GL_BLEND)

		glPushMatrix()
		glTranslate(-self.camPos[0],-self.camPos[1]-1,4)

		glPushMatrix()

		self.tileTex.setup()
		for i in xrange(5):
			for j in xrange(5):
				glPushMatrix()
				glTranslate(i,j,0)
				glColor4f(1,1,1,1)
				self.tmap[i+j*5].draw()
		#		self.drawmaptile(self.tmap[i+j*5])

				glPopMatrix()
		glPopMatrix()
	
		if self.editMode:
			self.editCursor.draw()
		else:
			self.hero.draw()
		glPopMatrix()

	def toggleEdit(self):
		self.editMode = not self.editMode
		if self.editMode:
			self.camTarget = self.editCursor
		else:
			self.camTarget = self.hero
	
	def update(self):
		td = Timer.diff
		###################
		# camera move
		cx,cy = self.camPos
		x,y = self.camTarget.pos

		dx = ((x-cx)**2)*2.0 * td
		dy = ((y-cy)**2)*2.0 * td

		if x-cx > 0:
			cx += dx
		else:
			cx-=dx
		if y-cy > 0:
			cy += dy
		else:
			cy-=dy

		self.camSpeed = 5*(dx**2+dy**2)
		self.camPos = cx,cy	
		####################
		
		if not self.editMode:
				self.hero.update(self)
		

	def drawOverlay(self):
		if self.editMode:
			self.editCursor.activeTile.drawEditOverlay()
		else:

			d = 1.0/64.0
			lq=self.hero.liquid
			self.liquidtex.setup()
			glBegin(GL_QUADS)
			glTexCoord((0+d),(0+d)); glVertex2f(50,480-50)
			glTexCoord((lq-d),(0+d)); glVertex2f( 50+100*lq,480-50)
			glTexCoord((lq-d),(1-d)); glVertex2f( 50+100*lq,480-70)
			glTexCoord((0+d),(1-d)); glVertex2f(50,480-70)
			glEnd()
			glColor3f(1,1,1)
			self.font.drawtext("liquid",50,640-32)

	def drawTile(self,verts,tilePos):
		glBegin(GL_QUADS)
		for p,t in izip(verts,self.genTexCoords(tilePos)):
			glTexCoord(*t); glVertex3f(*p)
		glEnd()

	def genTexCoords(self, (x,y)):
		D = 128.0/512.0
		x *= D
		y *= D
		d = 1.0/512.0 # shrink the uv by 'd' to prevent glitches
		return [ (x+d,y+D-d),(x+D-d,y+D-d),(x+D-d,y+d),(x+d,y+d) ]

	def loadTileset(self,mapfile):
		f = open(mapfile)
		lines = mf.readlines()
		f.close()

		for line in lines:
			line.strip()
			a = line.split()
			print a

	def saveLevel(self):
		print 'Saving level'
		slevel = levelData()
		slevel.tiles = []
		slevel.w = 5
		slevel.h = 5
		for t in self.tmap:
			slevel.tiles.append((t.id,t.solid))
		f = open('level1.lvl','w')
		pickle.dump(slevel,f)
		f.close()

	def loadLevel(self):
		print 'Loading level'
		try:
			f = open('level1.lvl','r')
			slevel = pickle.load(f)
			f.close()
		
			self.tmap = []
			for tid,sol in slevel.tiles:
				tile = Tile()
				tile.id = tid
				tile.solid = sol
				self.tmap.append(tile)
			self.w = slevel.w
			self.h = slevel.h
		except:
			print 'Loading failed'
			self.tmap = [Tile() for i in xrange(25)]
			self.w = 5
			self.h = 5


