
import pygame
from pygame.locals import *



from OpenGL.GL import *

from cTimer import Timer
from cGame import Game
from cScreen import Screen
from cMenu import Menu
from cSettings import Settings
from cInput import Input

from cJoy import Joystick


class App(Screen):
	def __init__(self):
		Screen.__init__(self)

		self.running = True
		self.settings = Settings()
		self.joystick = self.InitJoy()
		self.menu = Menu()
		self.game = Game()
		
		self.isatmenu = True
		self.menumap = [0,0] #value for checking if the user is currently mapping keys / joy

		#setup drawlists:
		self.drawlist = [self.game]
		self.drawlist.append(self.menu)
		
	def mainLoop(self):
		while self.running:
			evt = pygame.event.poll()
			if self.menumap[0]==2:
				Input.mapjoy(self)
			if evt.type == KEYDOWN:
				self.handleKeyDown(evt)
			
			self.game.update(self)
			
			glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
			self.draw()

			pygame.display.flip()
			
			if Timer.currentTime:
				tnow = pygame.time.get_ticks()
				diff = tnow - Timer.currentTime
				if diff < 10:
					pygame.time.wait(10-diff)
			Timer.update()
	
	def handleKeyDown(self,evt):
		if evt.key == K_ESCAPE:
			if(self.isatmenu):
				self.menu.active = len(self.menu.drawlist[self.menu.submenu].drawlist)-1
			if not self.isatmenu: 
				self.isatmenu = True;
				self.drawlist.append(self.menu)
				self.game.stop()
		if not self.isatmenu and evt.key == K_TAB:
			self.game.level.toggleEdit()


		if self.isatmenu:
			if evt.key == K_DOWN:
				if self.menumap[0]==0:
					self.menu.changemenu(1)
			if evt.key == K_UP:
				if self.menumap[0]==0:
					self.menu.changemenu(-1)
			
			if(self.menumap[0]==0):			
				val = self.menu.submenu,self.menu.active
				Menu.handleMenu2(self,val,evt.key)
			else:
				Input.mapkey(self,evt.key)
				#self.detectJoy()
				
	def InitJoy(self):
		pygame.joystick.init()
		
		joyful=Joystick()
		
		joyful.numjoy=pygame.joystick.get_count()
	
		
		if not joyful.numjoy == 0:
			joyful.setjoys(joyful.numjoy)
		return joyful
		
	def detectJoy(self):
		mjoy = self.settings.joy
		buttons=self.joystick.check(self.settings.joy,self.settings.keys)

if __name__ == '__main__':
	app = App()
	app.mainLoop()
	app.settings.savesettings("config.txt")
	app.game.level.saveLevel()

